<?php

class ArtikelController extends Controller
{
	/**
	 * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
	 * using two-column layout. See 'protected/views/layouts/column2.php'.
	 */
	public $layout='//layouts/column2';

	/**
	 * @return array action filters
	 */
	public function filters()
	{
		return array(
			'accessControl', // perform access control for CRUD operations
			'postOnly + delete', // we only allow deletion via POST request
		);
	}

	/**
	 * Specifies the access control rules.
	 * This method is used by the 'accessControl' filter.
	 * @return array access control rules
	 */
	public function accessRules()
	{
		return array(
			/*array('allow',  // allow all users to perform 'index' and 'view' actions
				'actions'=>array('index','view'),
				'users'=>array('*'),
			),*/
			array('allow', // allow authenticated user to perform ... actions
				'actions'=>array('create','update','admin','print','delete'),
				'users'=>array('@'),
			),
			array('deny',  // deny all users
				'actions'=>array('delete'),
                'users'=>array('*'),
			),
		);
	}

	/**
	 * Displays a particular model.
	 * @param integer $id the ID of the model to be displayed
	 */
	public function actionView($id)
	{
		$this->render('view',array(
			'model'=>$this->loadModel($id),
		));
	}
    
    /**
     * Liefert eine Druckliste alle Kundendaten + Artikel zurück
     */
    public function actionPrint()
    {
        $model=new Artikel('search');
        $model->unsetAttributes();  // clear any default values
        
        
        $user_id=Yii::app()->user->id;
        $model_user=null;
        if($user_id!=null)
            $model_user=User::model()->findbyPk($user_id);
        
        
        if(isset($_GET['Artikel']))
            $model->attributes=$_GET['Artikel'];

        $this->render('print',array(
            'model'=>$model,'model_user'=>$model_user,
        ));
    }

	/**
	 * Creates a new model.
	 * If creation is successful, the browser will be redirected to the 'view' page.
	 */
	public function actionCreate()
	{
		$model=new Artikel;

		// Uncomment the following line if AJAX validation is needed
		$this->performAjaxValidation($model);

		if(isset($_POST['Artikel']))
		{
			//setze userid für import, wenn nicht gefunden damit nicht gesetzt und damit error
            $user_id=Yii::app()->user->id;
            if($user_id!=null)
                $model->RefUserID=$user_id;
            
            //artikeldaten bereinigen um xxs etc zu unterbinden
            $_POST['Artikel']=Helper::cleanAllInputData($_POST['Artikel']);
            
            //artikeldaten manipulieren
            $_POST['Artikel']=Artikel::model()->manipulateArtikelData($_POST['Artikel']);
            
            //setze Erfassungszeitstempel
            $model->ErfassungTS=date("Y-m-d H:i:s");
            
            $model->attributes=$_POST['Artikel'];
            
            if($model->save())
            {
                $this->redirect(array('create'));    
            }
			//if($model->save())
				//$this->redirect(array('view','id'=>$model->ImportID));
		}

		$this->render('create',array(
			'model'=>$model,'userid'=>Yii::app()->user->id,
		));
	}

	/**
	 * Updates a particular model.
	 * If update is successful, the browser will be redirected to the 'view' page.
	 * @param integer $id the ID of the model to be updated
	 */
	public function actionUpdate($id)
	{
		//sicherstellen dass id nummerisch ist
        $id=intval($id);
        
        $model=$this->loadModel($id);

		// Uncomment the following line if AJAX validation is needed
		$this->performAjaxValidation($model);

		if(isset($_POST['Artikel']))
		{
		    //artikeldaten bereinigen um xxs etc zu unterbinden
            $_POST['Artikel']=Helper::cleanAllInputData($_POST['Artikel']);
            
            //artikeldaten manipulieren
            $_POST['Artikel']=Artikel::model()->manipulateArtikelData($_POST['Artikel']);
            
            $model->attributes=$_POST['Artikel'];
			if($model->save())
				$this->redirect(array('admin'));
		}

		$this->render('update',array(
			'model'=>$model,
		));
	}

	/**
	 * Deletes a particular model.
	 * If deletion is successful, the browser will be redirected to the 'admin' page.
	 * @param integer $id the ID of the model to be deleted
	 */
	public function actionDelete($id)
	{
		//id muss nummerisch sein
        $id=intval($id);
        
        //nur löschen wenn auch von diesem user erfasst
        $user_id_session=Yii::app()->user->id;
        $user_id_object=$this->loadModel($id)->RefUserID;
        if($user_id_session!=null && $user_id_object!=null && $user_id_session==$user_id_object)
        {
            $this->loadModel($id)->delete();         
        }
        else
        {
            throw new CHttpException(500,'Die ImportID:'.$id.' gehört nicht zu ihrem User, daher kann eine Löschung des Artikels nicht durchgeführt werden');    
        }
        
		// if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
		if(!isset($_GET['ajax']))
			$this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
	}

	/**
	 * Lists all models.
	 */
	public function actionIndex()
	{
		$dataProvider=new CActiveDataProvider('Artikel');
		$this->render('index',array(
			'dataProvider'=>$dataProvider,
		));
	}

	/**
	 * Manages all models.
	 */
	public function actionAdmin()
	{
		$model=new Artikel('search');
		$model->unsetAttributes();  // clear any default values
		if(isset($_GET['Artikel']))
			$model->attributes=$_GET['Artikel'];

		$this->render('admin',array(
			'model'=>$model,
		));
	}

	/**
	 * Returns the data model based on the primary key given in the GET variable.
	 * If the data model is not found, an HTTP exception will be raised.
	 * @param integer $id the ID of the model to be loaded
	 * @return Artikel the loaded model
	 * @throws CHttpException
	 */
	public function loadModel($id)
	{
		$model=Artikel::model()->findByPk($id);
		if($model===null)
			throw new CHttpException(404,'The requested page does not exist.');
		return $model;
	}

	/**
	 * Performs the AJAX validation.
	 * @param Artikel $model the model to be validated
	 */
	protected function performAjaxValidation($model)
	{
		if(isset($_POST['ajax']) && $_POST['ajax']==='artikel-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}
	}
    
    /**
    * Liefert den Warengruppennamen zu einer Warengruppenid in einer tabelle
    * 
    * @param mixed $data
    * @param mixed $row
    * @return mixed
    */
    protected function getWarengruppenName($data,$row)
    {     
        $model = Warengruppe::model()->findByPk($data->RefWarengruppeID);
        if(isset($model->Name) && isset($model->ID))
            return (string)$model->Name." (".$model->ID.")";
        else
            return "";
    }
    
    //beispiel von http://www.yiiframework.com/wiki/278/cgridview-render-customized-complex-datacolumns/
    //assume you have generated the Address model with gii too
    /**
    * Liefert Formatierten Preis für Ausdruck
    * 
    * @param mixed $data
    * @param mixed $row
    * @return mixed
    */
    protected function getFormatedPreis($data,$row)
    {     
        if(isset($data->Preis) && $data->Preis!="")
            return number_format ( $data->Preis , 2 , "," , "" )." €";
        else
            return "unbekannt, error?";
    }
}
